<?php
/*
 * General include file for SDS applications
 *
 * Copyright (c) 2002 Willy De la Court <wdl@linux-lovers.be>
 *
 */

//$SDS_LoginTemplate = array();
//$SDS_LoginTemplate = "/var/www/sds/templates/";
if( !empty($site_dir) ) {
    $site_dir .= "/";
}
$htdig_database = "/var/lib/htdig/" . $site_dir;
$htdig_config = $htdig_database . "htdig.conf";

require($_PHPLIB["libdir"] . "db_mysql.inc");
require($_PHPLIB["libdir"] . "ct_sql.inc");
require($_PHPLIB["libdir"] . "session.inc");
require($_PHPLIB["libdir"] . "auth.inc");
require($_PHPLIB["libdir"] . "perm.inc");
require($_PHPLIB["libdir"] . "user.inc");
require($_PHPLIB["libdir"] . "page.inc");
require($_PHPLIB["libdir"] . "template.inc");
require($_PHPLIB["libdir"] . "RSS_feed.php");

class SDS_CT_Sql extends CT_Sql {
	var $database_class = "DB_SDS";          ## Which database to connect...
	var $database_table = "active_sessions"; ## and find our session data in this table.
}

class SDS_Session extends Session {
	var $classname = "SDS_Session";

	var $cookiename     = "";                ## defaults to classname
	var $magic          = "HoCuSpOcUs";      ## ID seed
	var $mode           = "cookie";          ## We propagate session IDs with cookies
	var $fallback_mode  = "get";
	var $lifetime       = 0;                 ## 0 = do session cookies, else minutes
	var $that_class     = "SDS_CT_Sql";     ## name of data storage container class
	var $gc_probability = 5;
	var $allowcache     = "no";              ## "public", "private", or "no"
}

class SDS_User extends User {
	var $classname = "SDS_User";

	var $magic          = "AbRaCaDaBrA";     ## ID seed
	var $that_class     = "SDS_CT_Sql";     ## name of data storage container class
}

class SDS_Auth extends Auth {
	var $classname      = "SDS_Auth";

	var $lifetime       = 1440;

	var $database_class = "DB_SDS";
	var $database_table = "auth_user";

	function auth_loginform() {
		global $sess;
		global $_PHPLIB;
		global $SDS_LoginTemplate;

		include($SDS_LoginTemplate . "loginform.ihtml");
	}

	function auth_validatelogin() {
		global $username, $password;

		if(isset($username)) {
			$this->auth["uname"]=$username;        ## This provides access for "loginform.ihtml"
		}

		$uid = false;

		// Backwards compatibility select * instead of select user_id,perms,password,language
		$this->db->query(sprintf("select * ".
					"        from %s ".
					"       where username = '%s' ".
					"         and password = '%s'",
					$this->database_table,
					addslashes($username),
					addslashes($password)));

		while($this->db->next_record()) {
			$uid = $this->db->f("user_id");
			$this->auth["perm"] = $this->db->f("perms");
			$this->auth["language"] = $this->db->f("language");
			$this->auth["emp_grp"] = $this->db->f("emp_grp");
		}
		return $uid;
	}
}

class SDS_Challenge_Auth extends Auth {
	var $classname      = "SDS_Challenge_Auth";

	var $lifetime       =  1440;

	var $magic          = "SiMsAlAbIm";  ## Challenge seed
	var $database_class = "DB_SDS";
	var $database_table = "auth_user";

	function auth_loginform() {
		global $sess;
		global $challenge;
		global $_PHPLIB;
		global $SDS_LoginTemplate;

		$challenge = md5(uniqid($this->magic));
		$sess->register("challenge");

		include($SDS_LoginTemplate . "crloginform.ihtml");
	}

	function auth_validatelogin() {
		global $username, $password, $challenge, $response;

		if(isset($username)) {
			$this->auth["uname"]=$username;        ## This provides access for "loginform.ihtml"
		}
		// Backwards compatibility select * instead of select user_id,perms,password,language
		$this->db->query(sprintf("select * ".
				"from %s where username = '%s'",
					$this->database_table,
					addslashes($username)));

		if ($this->db->num_rows() == 0) {
			return false;
		}

		while($this->db->next_record()) {
			$uid   = $this->db->f("user_id");
			$perm  = $this->db->f("perms");
			$pass  = $this->db->f("password");
			$lang  = $this->db->f("language");
			$emp_grp = $this->db->f("emp_grp");
		}
		$expected_response = md5("$username:$pass:$challenge");

		## True when JS is disabled
		if ($response == "") {
			if ($password != $pass) {
				return false;
			} else {
				$this->auth["perm"] = $perm;
				$this->auth["language"] = $lang;
				$this->auth["emp_grp"] = $emp_grp;
				return $uid;
			}
		}

		## Response is set, JS is enabled
		if ($expected_response != $response) {
			return false;
		} else {
			$this->auth["perm"] = $perm;
			$this->auth["language"] = $lang;
			$this->auth["emp_grp"] = $emp_grp;
			return $uid;
		}
	}
}


class SDS_Challenge_Crypt_Auth extends Auth {
	var $classname      = "SDS_Challenge_Crypt_Auth";

	var $lifetime       =  1440;

	var $magic          = "FrObOzZiCa";  ## Challenge seed
	var $database_class = "DB_SDS";
	var $database_table = "auth_user_md5";

	function auth_loginform() {
		global $sess;
		global $challenge;
		global $_PHPLIB;
		global $SDS_LoginTemplate;

		$challenge = md5(uniqid($this->magic));
		$sess->register("challenge");

		include($SDS_LoginTemplate . "crcloginform.ihtml");
	}

	function auth_validatelogin() {
		global $username, $password, $challenge, $response;

		$this->auth["uname"]=$username;        ## This provides access for "loginform.ihtml"

		// Backwards compatibility select * instead of select user_id,perms,password,language
		$this->db->query(sprintf("select * ".
				"from %s where username = '%s'",
					$this->database_table,
					addslashes($username)));

		if ($this->db->num_rows() == 0) {
			return false;
		}

		while($this->db->next_record()) {
			$uid   = $this->db->f("user_id");
			$perm  = $this->db->f("perms");
			$pass  = $this->db->f("password");   ## Password is stored as a md5 hash
			$lang  = $this->db->f("language");
			$emp_grp = $this->db->f("emp_grp");
		}
		$expected_response = md5("$username:$pass:$challenge");

		## True when JS is disabled
		if ($response == "") {
			if (md5($password) != $pass) {       ## md5 hash for non-JavaScript browsers
				return false;
			} else {
				$this->auth["perm"] = $perm;
				$this->auth["language"] = $lang;
				$this->auth["emp_grp"] = $emp_grp;
				return $uid;
			}
		}

		## Response is set, JS is enabled
		if ($expected_response != $response) {
			return false;
		} else {
			$this->auth["perm"] = $perm;
			$this->auth["language"] = $lang;
			$this->auth["emp_grp"] = $emp_grp;
			return $uid;
		}
	}
}

class SDS_Default_Auth extends SDS_Challenge_Crypt_Auth {
	var $classname = "SDS_Default_Auth";

	var $nobody    = true;

	function auth_preauth() {
		return($this->auth_validatelogin());
	}
}

class SDS_Perm extends Perm {
	var $classname = "SDS_Perm";

	var $permissions = array(
				"user"       => 1,
				"author"     => 3,
				"editor"     => 7,
				"supervisor" => 15,
				"admin"      => 31
				);

	var $view_permissions = array(
				"(none)"     => "",
				"User"       => "user",
				"Author"     => "author",
				"Editor"     => "editor",
				"Supervisor" => "supervisor",
				"Admin"      => "admin"
				);

	function perm_invalid($does_have, $must_have) {
		global $perm, $auth, $sess;
		global $_PHPLIB;
		global $SDS_LoginTemplate;

		include($SDS_LoginTemplate . "perminvalid.ihtml");
	}

	function perm_opt($name, $current = "", $class = "") {
		reset($this->permissions);

		$ret = "<SELECT NAME=$name>\n";
		foreach( $this->permissions as $key => $value ) {
			$ret .= "<OPTION VALUE=\"$key\"";
			if( $key == $current ) {
				$ret .= " SELECTED";
			}
			$ret .= ">" . $key . "</OPTION>\n";
		}
		$ret .= "</SELECT>\n";
		return( $ret );
	}

	function view_opt() {
		global $permission_selectopt;

		reset($this->view_permissions);
		foreach( $this->view_permissions as $key => $value ) {
			$permission_selectopt->add_element($value, $key);
		}
	}
}

?>