/*
 *  TreeMenu Plugin Class
 *
 *  Copyright (c) 2002 Willy De la Court <wdl@linux-lovers.be>
 *
 */
?><?php
register_plugin("TreeMenu");

class PlainTreeMenuItem Extends PlainPlugin {
	var $elementcount = 0;
	var $text = "";
	var $url = "";
	var $pane = "";
	var $elementarr = array();

	/*********************************************/
	/* Set Info for this Menu Element            */
	/*********************************************/
	function PlainTreeMenuItem($text, $url = "", $pane = "") {
		$this->text = $text;
		$this->url = $url;
		$this->pane = $pane;
	}

	/*********************************************/
	/* Add a new Menu Element                    */
	/*********************************************/
	function add_element(& $obj) {
		$this->elementcount += 1;
		$this->elementarr[$this->elementcount] =& $obj;
	}

	/*********************************************/
	/* Gather Menu info in flat array            */
	/*********************************************/
	function getinfo(& $menu, $level = 0) {
		$menu->setinfo($level, $this->text, $this->url, $this->pane);
		foreach( array_keys($this->elementarr) as $k )
			$this->elementarr[$k]->getinfo($menu, $level + 1);
	}
}

class PlainTreeMenu Extends PlainTreeMenuItem {
	var $cnt = 0;
	var $result = "";
	var $tree = array();
	var $expand = array();
	var $levels = array();
	var $explevels = array();
	var $lastlevel = 0;
	var $maxlevel = 0;
	var $path = "";
	var $img_home     = "tree_home.gif";
	var $img_expand   = "tree_expand.gif";
	var $img_collapse = "tree_collapse.gif";
	var $img_line     = "tree_vertline.gif";
	var $img_split	  = "tree_split.gif";
	var $img_end      = "tree_end.gif";
	var $img_leaf     = "tree_leaf.gif";
	var $img_spc      = "tree_space.gif";
	var $img_fopen    = "tree_fopen.gif";
	var $img_fclosed  = "tree_fclosed.gif";


	function setinfo($level, $text, $url, $pane = "") {
		$this->tree[$this->cnt][0] = $level;
		$this->tree[$this->cnt][1] = $text;
		$this->tree[$this->cnt][2] = $url;
		$this->tree[$this->cnt][3] = $pane;
		$this->tree[$this->cnt][4] = 0;
		$this->cnt++;
		if($level > $this->maxlevel)
			$this->maxlevel = $level;
	}
	
	function setimagepath($path) {
		$this->path = $path;
	}

	function showtreemenu() {
		global $show, $p, $preg;

		/*********************************************/
		/* Clear all levels                          */
		/*********************************************/
		for($i = 0; $i < $this->cnt; $i++) {
			$this->expand[$i] = 0;
			$this->visible[$i] = 0;
			$this->levels[$i] = 0;
		}
		/*********************************************/
		/*  Get Node numbers to expand               */
		/*********************************************/
		if($p != "")
			$this->explevels = explode("|", $p);

		$i = 0;
		while($i < count($this->explevels))
			$this->expand[$this->explevels[$i++]] = 1;
		/*********************************************/
		/*  Find last nodes of subtrees              */
		/*********************************************/
		$this->lastlevel = $this->maxlevel;
		for($i = $this->cnt - 1; $i >= 0; $i--) {
			if($this->tree[$i][0] < $this->lastlevel) {
				for($j = $this->tree[$i][0]+1; $j <= $this->maxlevel; $j++)
					$this->levels[$j] = 0;
			}
			if($this->levels[$this->tree[$i][0]] == 0) {
				$this->levels[$this->tree[$i][0]] = 1;
				$this->tree[$i][4] = 1;
			}
			else
				$this->tree[$i][4] = 0;
			$this->lastlevel = $this->tree[$i][0];
		}

		/*********************************************/
		/*  Determine visible nodes                  */
		/*********************************************/

		// all root nodes are always visible
		for($i = 0; $i < $this->cnt; $i++)
			if ($this->tree[$i][0] <= 1)
				$this->visible[$i] = 1;

		for($i = 0; $i < count($this->explevels); $i++) {
			$n = $this->explevels[$i];
			if( ($this->visible[$n] == 1) && ($this->expand[$n] == 1) ) {
				$j = $n + 1;
				while( $this->tree[$j][0] > $this->tree[$n][0] ) {
					if($this->tree[$j][0] == $this->tree[$n][0]+1)
						$this->visible[$j] = 1;
					$j++;
				}
			}
		}

		/*********************************************/
		/*  Output nicely formatted tree             */
		/*********************************************/
		for($i = 0; $i < $this->maxlevel; $i++)
			$this->levels[$i] = 1;

		$this->maxlevel++;

		$this->result .= "<table cellspacing=0 cellpadding=0 border=0 cols=" . ($this->maxlevel + 3) . " width=100%>\n";
		//$this->result .= "<tr>";
		//for($i = 0; $i < $this->maxlevel; $i++)
		//	$this->result .= "<td class=treemenu width=16 nowrap></td>";
		//$this->result .= "<td class=treemenu width=100% nowrap>&nbsp;</td></tr>\n";
		$cntitems = 0;
		while($cntitems < $this->cnt) {
			if($this->visible[$cntitems]) {
				/****************************************/
				/* start new row                        */
				/****************************************/
				$this->result .= "<tr>";
				/****************************************/
				/* vertical lines from higher levels    */
				/****************************************/
				$i = 0;
				while($i < $this->tree[$cntitems][0] - 1) {
					if ($this->levels[$i] == 1)
						$this->result .= "<td class=treemenu nowrap width='16'><a name='" . $cntitems . "'></a><img src='" . $this->path . $this->img_line . "'></td>";
					else
						$this->result .= "<td class=treemenu nowrap width='16'><a name='" . $cntitems . "'></a><img src='" . $this->path . $this->img_spc . "'></td>";
					$i++;
				}

				/*********************************************************/
				/* corner at end of subtree or t-split not for Root Node */
				/*********************************************************/
				if($this->tree[$cntitems][0] != 0) {
					if ($this->tree[$cntitems][4] == 1) {
						$this->result .= "<td class=treemenu nowrap width='16'><img src='" . $this->path . $this->img_end . "'></td>";
						$this->levels[$this->tree[$cntitems][0] - 1] = 0;
					}
					else {
						$this->result .= "<td class=treemenu nowrap width='16'><img src='" . $this->path . $this->img_split . "'></td>";
						$this->levels[$this->tree[$cntitems][0] - 1] = 1;
					}
				}

				if($this->tree[$cntitems][0] == 0 ) {
					/********************************************/
					/* Root Node                                */
					/********************************************/
					$this->result .= "<td class=treemenu nowrap width='16'><img src='" . $this->path . $this->img_home . "'></td>";
				}
				else {
					/********************************************/
					/* Node (with subtree) or Leaf (no subtree) */
					/********************************************/
					if($this->tree[$cntitems + 1][0] > $this->tree[$cntitems][0]) {
						/****************************************/
						/* Create expand/collapse parameters    */
						/****************************************/
						$i = 0;
						$params = "";
						while($i < count($this->expand)) {
							if( ($this->expand[$i] == 1) && ($cntitems != $i) || ($this->expand[$i] == 0 && $cntitems == $i) )
								$params .= $i . "|";
							$i++;
						}

						if($this->expand[$cntitems] == 0) {
							$this->result .= "<td class=treemenu nowrap width='16'><a href='" . $PHP_SELF . "?p=" . $params . "#$cntitems'><img src='" . $this->path . $this->img_expand . "' border=no></a></td>";
							$this->result .= "<td class=treemenu><a href='" . $PHP_SELF . "?p=" . $params . "#$cntitems'><img src='" . $this->path . $this->img_fclosed . "' border=no></a></td>";
						} else {
							$this->result .= "<td class=treemenu nowrap width='16'><a href='" . $PHP_SELF . "?p=" . $params . "#$cntitems'><img src='" . $this->path . $this->img_collapse . "' border=no></a></td>";
							$this->result .= "<td class=treemenu><a href='" . $PHP_SELF . "?p=" . $params . "#$cntitems'><img src='" . $this->path . $this->img_fopen . "' border=no></a></td>";
						}
					}
					else {
						/*************************/
						/* Tree Leaf             */
						/*************************/
						if($this->tree[$cntitems][2] == "")
							$this->result .= "<td class=treemenu nowrap width='16'><img src='" . $this->path . $this->img_leaf . "' border=no></td>";
						else
							$this->result .= "<td class=treemenu nowrap width='16'><a href='" . $this->tree[$cntitems][2] . "' target='" . $this->tree[$cntitems][3] . "'><img src='" . $this->path . $this->img_leaf . "' border = no></a></td>";
					}
				}
				/****************************************/
				/* output item text                     */
				/****************************************/
				//	$this->result .= "<td class=treemenu nowrap colspan=" . ($this->maxlevel - $this->tree[$cntitems][0]) . " width='100%'><a href='" . $PHP_SELF . "?show=" . $show . "&p=$cntitems'>&nbsp;" . $this->tree[$cntitems][1] . "</a></td>";
				if($this->tree[$cntitems][2] == "")
					$this->result .= "<td class=treemenu nowrap colspan=" . ($this->maxlevel - $this->tree[$cntitems][0]) . " width='100%'>&nbsp;" . $this->tree[$cntitems][1] . "</td>";
				else
					$this->result .= "<td class=treemenu nowrap colspan=" . ($this->maxlevel - $this->tree[$cntitems][0]) . " width='100%'><a href='" . $this->tree[$cntitems][2] . "' target='" . $this->tree[$cntitems][3] . "'>&nbsp;" . $this->tree[$cntitems][1] . "</a></td>";

				/****************************************/
				/* end row                              */
				/****************************************/

				$this->result .= "</tr>\n";
			}
			$cntitems++;
		}
		$this->result .= "</table>\n";
		return $this->result;
	}

	function show() {
		$this->result = "";
		$this->cnt = 0;
		$this->maxlevel = 0;
		$this->getinfo($this);
		return $this->showtreemenu();
	}
}

class TreeMenu Extends PlainTreeMenu {
	var $classname = "TreeMenu";
	var $file = "treemenu.ihtml";

	function TreeMenu( $record = array()) {
		global $theme, $sess, $p, $preg, $PHP_SELF;

		$this->name = $record["name"];
		if( !isset($p) && isset($preg))
			$p = $preg;
		$preg = $p;
		$sess->register("preg");
		$this->path = "themes/images/";
		if( ! empty($record) ) {
			$this->title = $record["title"];
			$mdb = new DB_SDS;
			$mdb->query("select * from menuitems where name = '" . $this->name . "' order by menuorder");
			if( $mdb->num_rows() > 0 ) {
				$cnt = $mdb->num_rows();
				for( $i = 1; $i <= $cnt; $i++ ) {
					$mdb->next_record();
					$level = $mdb->f("level");
					$text = $mdb->f("menutext");
					$param = $mdb->f("param");
					if( !empty($param) )
						$page = $PHP_SELF . "?show=" . $mdb->f("param");
					else
						$page = $mdb->f("page");
					$this->setinfo($level, $text, $page);
				}
			}
		}
		$this->PlainPlugin($record);
	}

	function set_vars( $record ) {
		$this->vars["TITLE"] = $this->title;
		$this->vars["ADMIN"] = $this->admin_button();
		$this->vars["TEXT"] = $this->showtreemenu();
	}

	function show() {
		$t =& new Template($this->theme, "remove");
		$t->set_file(array("layout" => $this->file));
		$t->set_var($this->vars);
		$t->parse("out", array("layout"));
		return $t->get("out");
	}
}
